"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const commands_fs_1 = require("@seagull/commands-fs");
const aws_cdk_1 = require("aws-cdk");
const aws = require("../aws_sdk_handler");
const lib = require("../lib");
const provide_asset_folder_1 = require("../provide_asset_folder");
const seagull_app_1 = require("../seagull_app");
const set_aws_credentials_1 = require("../set_aws_credentials");
class SeagullProject {
    constructor(props) {
        this.account = props.account;
        this.appPath = props.appPath;
        this.branch = props.branch;
        this.stage = props.stage;
        this.profile = props.profile;
        this.region = props.region;
        this.pkgJson = require(`${this.appPath}/package.json`);
        const propsACMHandler = props.handlers && props.handlers.acmHandler;
        const propsCFHandler = props.handlers && props.handlers.cloudfrontHandler;
        const propsSTSHandler = props.handlers && props.handlers.stsHandler;
        this.acm = propsACMHandler || new aws.ACMHandler();
        this.cloudfront = propsCFHandler || new aws.CloudfrontHandler();
        this.sts = propsSTSHandler || new aws.STSHandler();
    }
    createSeagullApp() {
        return __awaiter(this, void 0, void 0, function* () {
            const name = this.getAppName();
            const sdk = new aws_cdk_1.SDK({});
            const account = this.account || (yield sdk.defaultAccount());
            const itemBucketName = yield this.getBucketName();
            const actions = [
                'sts:AssumeRole',
                'logs:*',
                'lambda:InvokeFunction',
                'lambda:InvokeAsync',
                'ses:*',
                's3:*',
                'events:*',
            ];
            const aliasConfig = yield aws.checkForAliasConfig(this.pkgJson, this.acm);
            const appProps = {
                addAssets: true,
                appPath: this.appPath,
                itemsBucket: itemBucketName,
                projectName: name,
                stackProps: { env: { account, region: this.region } },
            };
            yield addResources(this.appPath, itemBucketName);
            const app = new seagull_app_1.SeagullApp(appProps);
            const role = app.stack.addIAMRole('role', 'lambda.amazonaws.com', actions);
            app.role = role;
            const logBucketName = `logs-${appProps.stackProps.env.account}`;
            const env = yield getEnv(name, this.appPath, this.stage, logBucketName);
            const lambda = app.stack.addLambda('lambda', this.appPath, role, env);
            const apiGW = app.stack.addUniversalApiGateway('apiGW', lambda, this.stage);
            app.stack.addCloudfront('cloudfront', {
                aliasConfig,
                apiGateway: apiGW,
                logBucketName,
            });
            const s3DeploymentNeeded = this.stage === 'prod' || this.branch === 'master';
            const importS3 = () => app.stack.importS3(itemBucketName, role);
            const addS3 = () => app.stack.addS3(itemBucketName, role);
            s3DeploymentNeeded ? addS3() : importS3();
            app.stack.addLogGroup(`/aws/lambda/${name}-lambda-handler`);
            app.stack.addLogGroup(`/${name}/data-log`);
            const addCrons = this.stage === 'prod' || this.branch === 'master';
            const cronJson = addCrons ? yield buildCronJson(this.appPath) : [];
            const addRule = (rule) => app.stack.addEventRule(rule, lambda);
            cronJson.forEach(addRule);
            return app;
        });
    }
    customizeStack(app) {
        return __awaiter(this, void 0, void 0, function* () {
            const extensionPath = `${this.appPath}/infrastructure-aws.ts`;
            const hasExtensionFile = yield new commands_fs_1.FS.Exists(extensionPath).execute();
            return hasExtensionFile && (yield this.loadAndExecute(extensionPath, app));
        });
    }
    loadAndExecute(path, app) {
        return __awaiter(this, void 0, void 0, function* () {
            const extensionFkt = (yield Promise.resolve().then(() => require(`${path}`))).default;
            yield extensionFkt(app);
        });
    }
    getAppName() {
        const suffix = this.stage === 'test' ? `-${this.branch}-${this.stage}` : '';
        return `${this.pkgJson.name}${suffix}`.replace(/[^0-9A-Za-z-]/g, '');
    }
    deployProject() {
        return __awaiter(this, void 0, void 0, function* () {
            this.validate();
            const app = yield this.createSeagullApp();
            yield this.customizeStack(app);
            yield app.deployStack();
            const url = (yield aws.getCFURL(this.getAppName(), this.cloudfront)) || '';
            yield new commands_fs_1.FS.WriteFile('/tmp/cfurl.txt', url).execute();
            console.info('cloudfront-url', url);
        });
    }
    destroyProject() {
        return __awaiter(this, void 0, void 0, function* () {
            this.validate();
            const app = yield this.createSeagullApp();
            yield app.destroyStack();
        });
    }
    diffProject() {
        return __awaiter(this, void 0, void 0, function* () {
            const app = yield this.createSeagullApp();
            yield this.customizeStack(app);
            yield app.diffStack();
        });
    }
    validate() {
        const hasValidProfile = set_aws_credentials_1.setCredsByProfile(this.profile);
        !hasValidProfile && lib.noCredentialsSet();
    }
    getBucketName() {
        return __awaiter(this, void 0, void 0, function* () {
            const accountId = yield aws.getAccountId(this.sts);
            const prefix = `${this.region}-${accountId}-`;
            const bucketName = `${require(`${this.appPath}/package.json`).name}-items`;
            const suffix = `${this.stage !== 'prod' ? `-${this.stage}` : ''}`;
            return `${prefix}${bucketName}${suffix}`;
        });
    }
}
exports.SeagullProject = SeagullProject;
function addResources(appPath, itemsBucket) {
    return __awaiter(this, void 0, void 0, function* () {
        const provideAssetFolder = new provide_asset_folder_1.ProvideAssetFolder(appPath);
        yield provideAssetFolder.execute();
        itemsBucket && (yield replaceS3BucketName(appPath, itemsBucket));
    });
}
function replaceS3BucketName(appPath, itemsBucket) {
    return __awaiter(this, void 0, void 0, function* () {
        const lambdaPath = `.seagull/deploy/dist/assets/backend/lambda.js`;
        const lambdaFile = `${appPath}/${lambdaPath}`;
        const lambda = yield new commands_fs_1.FS.ReadFile(lambdaFile).execute();
        const lambdaWithBucketName = lambda.replace('demo-bucket', itemsBucket);
        yield new commands_fs_1.FS.WriteFile(lambdaFile, lambdaWithBucketName).execute();
    });
}
function buildCronJson(appPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const cronPath = `${appPath}/dist/cron.json`;
        const cronFile = yield new commands_fs_1.FS.ReadFile(cronPath).execute();
        return cronFile && cronFile !== '' ? JSON.parse(cronFile) : [];
    });
}
function getEnv(name, appPath, stage, logBucket) {
    return __awaiter(this, void 0, void 0, function* () {
        const env = {
            APP: name,
            LOG_BUCKET: `${name}-${logBucket}`,
            MODE: 'cloud',
            NODE_ENV: 'production',
            STAGE: stage,
        };
        const configPath = `${appPath}/.env.${stage}`;
        const config = yield new commands_fs_1.FS.ReadFile(configPath).execute();
        return lib.addEnvFromFile(env, config);
    });
}
//# sourceMappingURL=seagull_project.js.map